# --------------------------------------------------------
#
# General remarks
#
# For computing the bimodality coefficient (BC), we need
# functions for skewness and kurtosis. These are not built
# into R but several packages offer convenient functions
# (e.g., 'e1071', 'moments', 'PerformanceAnalytics', and
# 'psych'). These packages have different algorithms for
# computing both measures and we recommend using 'e1071'
# that (a) allows for specifiying which algorithm is used
# and (b) gives considerable detail about the implemented
# algorithms.
#
# Thus, after having loaded the package with

library(e1071)

# we can compute unbiased estimates for skewness and kur-
# tosis with the input argument "type = 2". When we assume
# the datastored in a numeric vector of the name 'data', 
# the BC can be computed in the following four steps:
#
#    m3 = skewness(data,type = 2)
#    m4 = kurtosis(data,type = 2)
#    n  = length(data)
#    bc = (m3^2+1) / (m4 + 3 * ( (n-1)^2 / ((n-2)*(n-3)) ))
#
# This procedure is implemented for the data of figure A-F
# of the article in the following.
#
# --------------------------------------------------------



# --------------------------------------------------------
#
# Fig. A-D

# Define (x-) values.
x <- 1:11

# Define frequencies (instead of writing the entire data).
y <- matrix(data=NA,nrow=6,ncol=11)
y[1,] = c( 3,  5,  5, 10, 17, 20, 17, 10,  5,  5,  3)
y[2,] = c( 2, 26, 14,  6,  2,  0,  2,  6, 14, 26,  2)
y[3,] = c( 2,  3,  3,  3,  3,  4,  5, 11, 21, 41,  4)
y[4,] = c( 2,  3,  6, 17,  3,  4,  5, 12, 14, 30,  4)

# Prepare analysis
m3 <- matrix(data=NA,nrow=1,ncol=4)
m4 <- matrix(data=NA,nrow=1,ncol=4)
n  <- matrix(data=NA,nrow=1,ncol=4)
bc <- matrix(data=NA,nrow=1,ncol=4)

# Prepare plots
plots <- matrix(1:4, 2,2, byrow=TRUE)
layout(plots)
layout.show(4)
par(mar = c(3, 5, 2, 2))

# Process the data corresponding to each figure
for (ifig in 1:4) {
  
  # Transform frequencies in an actual vector of raw data
  data <- NULL
  for (i in 1:length(x)) {
    data <- c(data, rep.int(x[i],y[ifig,i]))
  }

  # Compute BC
  m3[ifig] = skewness(data,type=2)
  m4[ifig] = kurtosis(data,type=2)
  n[ifig]  = length(data);
  bc[ifig] = (m3[ifig]^2+1) / (m4[ifig] + 3 * 
             ( (n[ifig]-1)^2 / ((n[ifig]-2) * 
             (n[ifig]-3)) ));
  
  # Plot histogram
  hist(data,breaks=c(0:12),include.lowest=FALSE,
       ylim=c(0,47),xlim=c(0,11),
       main=paste("BC = ", round(bc[ifig],2)))
}

# Clean up workspace
rm(data)
rm(i)
rm(ifig)
rm(x)

# Display BCs
bc
# --------------------------------------------------------